
.define Mem_Ctrl $3e
.define IO_Ctrl $3f
.define VCounter $7e
.define HCounter $7f
.define VDP_Data $be
.define VDP_Ctrl $bf
.define IO_PortA $dc
.define IO_PortB $dd

.define NMI_row			$c800
.define HVScroll		$c802
.define Timer1			$c804
.define Timer2			$c806
.define stdout_col		$c810
.define stdout_row		$c811
.define KEY1_old		$c812
.define KEY1_pressed	$c813
.define KEY1_released	$c814
.define KEY1_current	$c815
.define MENU_POS		$c816
.define MENU_POS_OLD	$c817
.define TEMP_REGISTERS	$c818
.define TestResult01	$c900
.define TestResult02	$c901
.define TestResult03	$c902
.define TestResult04	$c903
.define TestResult05	$c904
.define TestResult06	$c905
.define TestResult07	$c906
.define TestResult08	$c907
.define TestResult09	$c908
.define TestResult10	$c909
.define TestResult11	$c90a
.define TestResult12	$c90b
.define TestResult13	$c90c
.define TestResult14	$c90d
.define TestResult15	$c90e
.define TestResult16	$c90f
.define TestResult17	$c910
.define TestResult18	$c911
.define TestResult19	$c912
.define TestResult20	$c913
.define TestResult21	$c914
.define TestResult22	$c915
.define TestResult23	$c916
.define TestResult24	$c917
.define TestResult25	$c918
.define TestResult26	$c919
.define TestResult27	$c91A
.define TestResult28	$c91B
.define TestResult29	$c91C
.define TestResult30	$c91D
.define TestAreaHC		$ca00
.define TestAreaVC		$cc00
.define RegisterArea	$dc00

.define NameTableAddress    $3800   ; must be a multiple of $800; usually $3800; fills $700 bytes (unstretched)
.define SpriteTableAddress  $3f00   ; must be a multiple of $100; usually $3f00; fills $100 bytes
.define VDP_Mode_Read		$0000	;Read mode for the VDP
.define VDP_Mode_Write		$4000	;Write mode for the VDP
.define VDP_Mode_Register	$8000	;Register mode for the VDP
.define VDP_Mode_Palette	$C000	;Palette mode for the VDP

.define STDOUT_START_COL	$01		;can be changed to handle GG and other things.
.define STDOUT_START_ROW	$00		;can be changed to handle GG and other things.
.define STDOUT_END_COL		$20		;can be changed to handle GG and other things.
.define STDOUT_END_ROW		$18		;can be changed to handle GG and other things.

.define KEY_UP			$01
.define KEY_DOWN		$02
.define KEY_LEFT		$04
.define KEY_RIGHT		$08
.define KEY_1			$10
.define KEY_2			$20
;==============================================================
; WLA-DX banking setup
;==============================================================
.memorymap
defaultslot 0
slotsize $4000
slot 0 $0000
slot 1 $4000
slot 2 $8000
.endme


.rombankmap
bankstotal 2
banksize $4000
banks 2
.endro

; Memory map (for Phantasy Star tilemap decompression)
.enum $c000
TileMapData: dsb 32*24*2

.ende

;==============================================================
; SDSC tag and SMS rom header
;==============================================================
.sdsctag 1.20,"SMSVDPTest","Test the SMS VDP","FluBBa"

.bank 0 slot 0
.org $0000

.include "colours.inc"
.include "Phantasy Star decompressors.inc"

;==============================================================
; Boot section
;==============================================================
.org $0000
.section "Boot section" force
ColdReset:
	di				; disable interrupts
	im 1			; Interrupt mode 1
	jp DumpAllRegisters
;	jp main			; jump to main program
.ends

;==============================================================
; IRQ handler
;==============================================================
.org $0038
.section "IRQ handler" force
	; Only acknowledge VDP irq, 13 cycles
	push af						;11 cyc
	in a,(VDP_Ctrl)				;11 cyc
	and $80
	jr Z,doHBL

	push hl						;11 cyc
	ld hl,(Timer1)
	inc hl
	ld (Timer1),hl
	pop hl						;10 cyc
	pop af						;10 cyc
	ei							;4 cyc
	reti						;14 cyc
doHBL
	ld a,(Timer2)
	inc a
	ld (Timer2),a
noVBL
	pop af						;10 cyc
	ei							;4 cyc
	reti						;14 cyc
.ends

;==============================================================
; Pause button handler, NMI
;==============================================================
.org $0066
.section "Pause button handler" force

	push af						;11 cyc
	in a,(VCounter)				;11 cyc
	ld (NMI_row),a

	ld a,$14					; mode4
	out (VDP_Ctrl),a
	ld a,$80
	out (VDP_Ctrl),a
	pop af						;10 cyc

	retn						;14
.ends

;==============================================================
; Main program
;==============================================================
.section "Main program" free
WarmReset:
	di
	im 1
main:
	ld sp, $dff0

	; Initialise VDP
	call VDP_DefaultInitialise
	;call VDP_ClearVRAM

	call VDP_SpritesOff ; they mess things up


	; Load palette
	ld hl,$c000					; palette index 0 write address
	call VDP_VRAMToHL
	ld hl,charpalette				; data
	ld bc,charpalettesize			; size
	call VDP_WriteToVRAM

	; Load tiles
	ld de,$4000
	ld hl,chartiles
	call LoadTiles4BitRLENoDI

	call StdOut_Init

	ld hl,menu_text
	call StdOut_Write
	call VDP_ScreenOn
	call WaitForNoButton
	ld a,$00
	ld (KEY1_current),a	;init old keys.

MenuStart:
	ld a,$00
	ld (MENU_POS),a		;init menu pos
	ld (MENU_POS_OLD),a	;init menu pos
	ei
-:	halt
	call ReadKeys
	ld a,(MENU_POS)
	ld (MENU_POS_OLD),a
	ld d,a
	ld a,(KEY1_pressed)
	cp KEY_1
	jr z,RunMenu
	cp KEY_UP
	jr nz,+
	dec d
+:
	cp KEY_DOWN
	jr nz,+
	inc d
+:
	ld a,d
	cp $FF
	jr nz,+
	ld a,$00
+:
	cp $04
	jr nz,+
	ld a,$03
+:
	ld (MENU_POS),a

	ld a,$01
	ld (stdout_col),a
	ld a,(MENU_POS_OLD)
	add a,$03
	ld (stdout_row),a
	call StdOut_SetVRAMPointer
	ld hl,space_text
	call StdOut_Write
	
	ld a,$01
	ld (stdout_col),a
	ld a,(MENU_POS)
	add a,$03
	ld (stdout_row),a
	call StdOut_SetVRAMPointer
	ld hl,marker_text
	call StdOut_Write

	jr -

RunMenu:
	ld a,(MENU_POS)
	cp $00
	jr z,DoSMSTest
	cp $01
	jr z,DoMDTest
	cp $02
	jr z,DoMDPaletteTest
	cp $03
	jr z,DoShowRegisters
	jp MenuStart

ReadKeys:
	ld a,(KEY1_current)
	ld (KEY1_old),a
	ld d,a			;save old keys
	in a,(IO_PortA) ; get input
	cpl             ; invert bits
	ld (KEY1_current),a
	ld e,a
	xor d			;which keys have changed
	ld d,a			;save a
	and e			;mask out keys currently pressed
	ld (KEY1_pressed),a
	ld a,e
	cpl
	ld e,a
	ld a,d
	and e
	ld (KEY1_released),a

	ret

DoSMSTest:
	di              ; disable interrupts
	call VDP_ScreenOff
	call StdOut_Cls
	jp StartSMSTest

DoMDTest:
	di              ; disable interrupts
	call VDP_ScreenOff
	call StdOut_Cls
	jp StartMDTest

DoMDPaletteTest:
	di              ; disable interrupts
	call VDP_ScreenOff
	call StdOut_Cls
	jp StartMDPaletteTest

DoShowRegisters:
	di              ; disable interrupts
	call VDP_ScreenOff
	call StdOut_Cls
	jp StartShowRegisters

;==============================================================
StartSMSTest:
	ld hl,sms_test1_text
	call StdOut_Write

	call RunTest1
	call RunTest2
	call RunTest3
	call RunTest4
	call RunTest5
	call RunTest6
	call RunTest7
	call RunTest8
	call RunTest9
	call RunTest10
	call RunTest11
	call RunTest12
	call RunTest13
	call RunTest14
	call RunTest15
	call RunTest16
	call RunTest17
	call RunTest18
	call RunTest19

	call StdOut_SetVRAMPointer

	ld hl,res_text_1
	ld a,(TestResult01)
	call WriteResult
	ld hl,res_text_2
	ld a,(TestResult02)
	call WriteResult
	ld hl,res_text_3
	ld a,(TestResult03)
	call WriteResult
	ld hl,res_text_4
	ld a,(TestResult04)
	call WriteResult
	ld hl,res_text_5
	ld a,(TestResult05)
	call WriteResult
	ld hl,res_text_6
	ld a,(TestResult06)
	call WriteResult
	ld hl,res_text_7
	ld a,(TestResult07)
	call WriteResult
	ld hl,res_text_8
	ld a,(TestResult08)
	call WriteResult
	ld hl,res_text_9
	ld a,(TestResult09)
	call WriteResult
	ld hl,res_text_10
	ld a,(TestResult10)
	call WriteResult
	ld hl,res_text_11
	ld a,(TestResult11)
	call WriteResult
	ld hl,res_text_12
	ld a,(TestResult12)
	call WriteResult
	ld hl,res_text_13
	ld a,(TestResult13)
	call WriteResult
	ld hl,res_text_14
	ld a,(TestResult14)
	call WriteResult
	ld hl,res_text_15
	ld a,(TestResult15)
	call WriteResult
	ld hl,res_text_16
	ld a,(TestResult16)
	call WriteResult
	ld hl,res_text_17
	ld a,(TestResult17)
	call WriteResult
	ld hl,res_text_18
	ld a,(TestResult18)
	call WriteResult
	ld hl,res_text_19
	ld a,(TestResult19)
	call WriteResult

	
	ld hl,continue_text
	call StdOut_Write
	call VDP_ScreenOn

	ei
	call WaitForButton
;==============================================================
SMSTest2:
	di              ; disable interrupts
	call VDP_ScreenOff
	call StdOut_Cls
	ld hl,sms_test2_text
	call StdOut_Write

	call RunTest21
	call RunTest22
	call RunTest23
	call RunTest24
	call RunTest25
	call RunTest26
	call RunTest27
	call RunTest28
	call RunTest29


	call StdOut_SetVRAMPointer

	ld hl,res_text_21
	ld a,(TestResult21)
	call WriteResult_VCount
	ld hl,res_text_22
	ld a,(TestResult22)
	call WriteResult
	ld hl,res_text_23
	ld a,(TestResult23)
	call WriteResult
	ld hl,res_text_24
	ld a,(TestResult24)
	call WriteResult
	ld hl,res_text_25
	ld a,(TestResult25)
	call WriteResult
	ld hl,res_text_26
	ld a,(TestResult26)
	call WriteResult
	ld hl,res_text_27
	ld a,(TestResult27)
	call WriteResult
	ld hl,res_text_28
	ld a,(TestResult28)
	call WriteResult
	ld hl,res_text_29
	ld a,(TestResult29)
	call WriteResult

	call VDP_ScreenOn

	ei
	call WaitForButton
	jp WarmReset
;==============================================================
StartMDTest:
	ld hl,md_test1_text
	call StdOut_Write

	call RunTest1
	call RunTest2
	call RunTest3
	call RunTest4
	call RunTest5
	call RunTest6_MD
	call RunTest7
	call RunTest8
	call RunTest9_MD
	call RunTest10_MD
	call RunTest11_MD
	call RunTest12_MD
	call RunTest13
	call RunTest14
	call RunTest15
	call RunTest16
	call RunTest17
	call RunTest18
	call RunTest19

	call StdOut_SetVRAMPointer

	ld hl,res_text_1
	ld a,(TestResult01)
	call WriteResult
	ld hl,res_text_2
	ld a,(TestResult02)
	call WriteResult
	ld hl,res_text_3
	ld a,(TestResult03)
	call WriteResult
	ld hl,res_text_4
	ld a,(TestResult04)
	call WriteResult
	ld hl,res_text_5
	ld a,(TestResult05)
	call WriteResult
	ld hl,res_text_6
	ld a,(TestResult06)
	call WriteResult
	ld hl,res_text_7
	ld a,(TestResult07)
	call WriteResult
	ld hl,res_text_8
	ld a,(TestResult08)
	call WriteResult
	ld hl,res_text_9_md
	ld a,(TestResult09)
	call WriteResult
	ld hl,res_text_10_md
	ld a,(TestResult10)
	call WriteResult
	ld hl,res_text_11_md
	ld a,(TestResult11)
	call WriteResult
	ld hl,res_text_12_md
	ld a,(TestResult12)
	call WriteResult
	ld hl,res_text_13
	ld a,(TestResult13)
	call WriteResult
	ld hl,res_text_14
	ld a,(TestResult14)
	call WriteResult
	ld hl,res_text_15
	ld a,(TestResult15)
	call WriteResult
	ld hl,res_text_16
	ld a,(TestResult16)
	call WriteResult
	ld hl,res_text_17
	ld a,(TestResult17)
	call WriteResult
	ld hl,res_text_18
	ld a,(TestResult18)
	call WriteResult
	ld hl,res_text_19
	ld a,(TestResult19)
	call WriteResult

	
	call VDP_ScreenOn

	ei
	call WaitForButton
;==============================================================
MDTest2:
	di              ; disable interrupts
	call VDP_ScreenOff
	call StdOut_Cls
	ld hl,md_test2_text
	call StdOut_Write

	call RunTest21
	call RunTest22
	call RunTest23
	call RunTest24
	call RunTest25_MD
	call RunTest26
	call RunTest27
	call RunTest28
	call RunTest29


	call StdOut_SetVRAMPointer

	ld hl,res_text_21
	ld a,(TestResult21)
	call WriteResult_VCount
	ld hl,res_text_22
	ld a,(TestResult22)
	call WriteResult
	ld hl,res_text_23
	ld a,(TestResult23)
	call WriteResult
	ld hl,res_text_24
	ld a,(TestResult24)
	call WriteResult
	ld hl,res_text_25
	ld a,(TestResult25)
	call WriteResult
	ld hl,res_text_26
	ld a,(TestResult26)
	call WriteResult
	ld hl,res_text_27
	ld a,(TestResult27)
	call WriteResult
	ld hl,res_text_28
	ld a,(TestResult28)
	call WriteResult
	ld hl,res_text_29
	ld a,(TestResult29)
	call WriteResult

	call VDP_ScreenOn
	ei
	call WaitForButton
;	jp WarmReset
;==============================================================
MDTest3:
	di              ; disable interrupts
	call VDP_ScreenOff
	call StdOut_Cls
	ld hl,md_page3_text
	call StdOut_Write

	ld hl,TestAreaHC
	ld b,229
	call StdOut_Write_Hex_Bytes


	call VDP_ScreenOn
	ei
	call WaitForButton
	jp WarmReset
;==============================================================
StartShowRegisters:
;==============================================================
	call VDP_ScreenOff
	call StdOut_Cls
	ld hl,cpu_regs_text
	call StdOut_Write


	ld hl,reg_text_sp
	ld de,(RegisterArea+$00)
	call WriteRegisterValue
	ld hl,reg_text_ir
	ld de,(RegisterArea+$02)
	call WriteRegisterValue
	ld hl,reg_text_af
	ld de,(RegisterArea+$04)
	call WriteRegisterValue
	ld hl,reg_text_bc
	ld de,(RegisterArea+$06)
	call WriteRegisterValue
	ld hl,reg_text_de
	ld de,(RegisterArea+$08)
	call WriteRegisterValue
	ld hl,reg_text_hl
	ld de,(RegisterArea+$0A)
	call WriteRegisterValue
	ld hl,reg_text_ix
	ld de,(RegisterArea+$0C)
	call WriteRegisterValue
	ld hl,reg_text_iy
	ld de,(RegisterArea+$0e)
	call WriteRegisterValue
	ld hl,reg_text_af2
	ld de,(RegisterArea+$10)
	call WriteRegisterValue
	ld hl,reg_text_bc2
	ld de,(RegisterArea+$12)
	call WriteRegisterValue
	ld hl,reg_text_de2
	ld de,(RegisterArea+$14)
	call WriteRegisterValue
	ld hl,reg_text_hl2
	ld de,(RegisterArea+$16)
	call WriteRegisterValue

	call VDP_ScreenOn
	ei
	call WaitForButton
	jp WarmReset
;==============================================================
.ends

;==============================================================
; Set Border Color.
;==============================================================
; A=color, Clobbers A & d
;==============================================================
.section "Set Bd color" free
VDP_SetBdColor:
;set palette 0
  ld d,a						;4 cyc
  ld a,$10						;7 cyc
  out (VDP_Ctrl),a				;11 cyc
  ld a,$c0						;7 cyc
  out (VDP_Ctrl),a				;11 cyc

  ld a,d						;4 cyc
  out (VDP_Data),a				;11 cyc

  ret							;10 cyc
.ends							;65 cycles total

;==============================================================
; Set Background Color.
;==============================================================
; A=color, Clobbers A & d
;==============================================================
.section "Set Bg color" free
VDP_SetBgColor:
;set palette 0
  ld d,a						;4 cyc
  ld a,$00						;7 cyc
  out (VDP_Ctrl),a				;11 cyc
  ld a,$c0						;7 cyc
  out (VDP_Ctrl),a				;11 cyc

  ld a,d						;4 cyc
  out (VDP_Data),a				;11 cyc

  ret							;10 cyc
.ends							;65 cycles total

;==============================================================
; Set up VDP registers (default values)
;==============================================================
; Call DefaultInitialiseVDP to set up VDP to default values.
; Also defines NameTableAddress, SpriteTableAddress and SpriteSet
; which can be used after this code in the source file.
; To change the values used, copy and paste the modified data
; and code into the main source. Data is commented to help.
;==============================================================
.section "Initialise VDP to defaults" free
VDP_DefaultInitialise:
    push hl
    push bc
        ld hl,_Data
        ld b,_End-_Data
        ld c,VDP_Ctrl
        otir
    pop bc
    pop hl
    ret

.define SpriteSet           0       ; 0 for sprites to use tiles 0-255, 1 for 256+

_Data:
    .db %00000100,$80
    ;    |||||||`- Disable sync
    ;    ||||||`-- M2, Enable extra height modes
    ;    |||||`--- SMS mode instead of SG
    ;    ||||`---- Shift sprites left 8 pixels
    ;    |||`----- Enable line interrupts
    ;    ||`------ Blank leftmost column for scrolling
    ;    |`------- Fix top 2 rows during horizontal scrolling
    ;    `-------- Fix right 8 columns during vertical scrolling
    .db %00100000,$81
    ;     |||||||`- Zoomed sprites -> 16x16 pixels
    ;     ||||||`-- Doubled sprites -> 2 tiles per sprite, 8x16
    ;     |||||`--- MD mode instead of SMS
    ;     |||`---- 30 row/240 line mode
    ;     ||`----- 28 row/224 line mode
    ;     |`------ Enable VBlank interrupts
    ;     `------- Enable display

    .db (NameTableAddress>>10) |%11110001,$82
	.db $FF,$83
	.db %00000011,$84
    .db (SpriteTableAddress>>7)|%10000001,$85
    .db (SpriteSet<<2)         |%11111011,$86
    .db $00,$87
    ;    `-------- Border palette colour (sprite palette)
    .db $00,$88
    ;    ``------- Horizontal scroll
    .db $00,$89
    ;    ``------- Vertical scroll
    .db $FF,$8a
    ;    ``------- Line interrupt spacing ($ff to disable)
_End:
.ends

;==============================================================
; Clear VRAM
;==============================================================
; Sets all of VRAM to zero
;==============================================================
.section "Clear VRAM" free
VDP_ClearVRAM:
	push af
	push hl
		ld hl,$0000|VDP_Mode_Write
		call VDP_VRAMToHL
		; Output 16KB of zeroes
		ld hl, $4000     ; Counter for 16KB of VRAM
		ld a,$00         ; Value to write
		call VDP_FillVRAM
	pop hl
	pop af
	ret
.ends

;==============================================================
;Fill VRAM
;==============================================================
; Fills a section of VRAM to with contents of a
; hl number of bytes
;==============================================================
.section "Fill VRAM" free
VDP_FillVRAM:
	push bc
		ld b,a
-:		ld a,b				; Value to write
		out (VDP_Data),a	; Output to VRAM address, which is auto-incremented after each write
		dec hl
		ld a,h
		or l
		jp nz,-
	pop bc
	ret
.ends

;==============================================================
; VRAM to HL
;==============================================================
; Sets VRAM write address to hl
;==============================================================
.section "VRAM to HL" free
VDP_VRAMToHL:
	push af
		ld a,l
		out (VDP_Ctrl),a
		ld a,h
		out (VDP_Ctrl),a
	pop af
	ret
.ends

;==============================================================
; VRAM writer
;==============================================================
; Writes BC bytes from HL to VRAM
; Clobbers HL, BC, A
;==============================================================
.section "Raw VRAM writer" free
VDP_WriteToVRAM:
-:	ld a,(hl)
	out (VDP_Data),a
	inc hl
	dec bc
	ld a,c
	or b
	jp nz,-
	ret
VDP_WriteToVRAM_Short:
-:	ld a,(hl)
	out (VDP_Data),a
	inc hl
	djnz -
	ret
.ends

;==============================================================
; VRAM text writer
;==============================================================
; Writes text from HL to VRAM, stops at a 0.
; Clobbers HL, A
;==============================================================
.section "SdtOut writer" free
StdOut_Write:
-:
	ld a,(hl)
	inc hl
	or a
	jp z,+
	call StdOut_Char
	jr -
+:
	ret
StdOut_Char:
	or a
	ret z
	cp $0a
	jp z,StdOut_NewLine
	sub 32
StdOut_Raw:
	ex af,af'
-:	ld a,(stdout_col)
	inc a
	ld (stdout_col),a
	cp STDOUT_END_COL
	jr nz,+
	call z,StdOut_NewLine
	jr -
+:	ex af,af'
	out (VDP_Data),a
	ld a,0
	out (VDP_Data),a
	ret
.ends

;==============================================================
; VRAM hex writer
;==============================================================
; Writes hex from HL to VRAM until b=0.
; Clobbers HL, B, A
;==============================================================
.section "SdtOut hex writer" free
StdOut_Write_Hex:
-:	ld a,(hl)
	inc hl
	call StdOut_Write_Hex_Reg_A
	djnz -
	ret

StdOut_Write_Hex_Bytes:
-:	ld a,(hl)
	inc hl
	call StdOut_Write_Hex_Reg_A
	ld a,0
	call StdOut_Raw
	djnz -
	ret

StdOut_Write_Hex_Reg_A:
-:
	push de
	ld d,a
	srl a
	srl a
	srl a
	srl a
	cp $0A
	jr C,+
	add a,$07
+:
	add a,$10
	call StdOut_Raw

	ld a,d
	and $0F
	cp $0A
	jr C,+
	add a,$07
+:
	add a,$10
	pop de
	jp StdOut_Raw
.ends



;==============================================================
.section "StdOut Misc" free
;==============================================================
; StdOut Init
;==============================================================
StdOut_Init:
;	jp StdOut_Cls

;==============================================================
; StdOut Clear Screen
;==============================================================
StdOut_Cls:
	call StdOut_Home
	ld hl,$600		; Counter for 32x24 Tilemap
	ld a,$00		; Value to write
	call VDP_FillVRAM

;==============================================================
; StdOut Reset VRAM Pointer
;==============================================================
StdOut_Home:
	push bc
	ld c,STDOUT_START_COL		;reset col
	ld b,STDOUT_START_ROW		;reset row
	ld (stdout_col),bc
	pop bc
;==============================================================
; Set VRAM pointer to stdout col/row
;==============================================================
StdOut_SetVRAMPointer:
	push bc
	push hl
	ld bc,(stdout_col)
	ld a,b
	sla a
	sla a
	sla a
	sla a
	sla a
	or c
	sla a
	ld c,a
	ld a,b
	srl a
	srl a
	ld b,a
	ld hl,NameTableAddress | VDP_Mode_Write
	add hl,bc
	call VDP_VRAMToHL
	pop hl
	pop bc
	ret
.ends

;==============================================================
; StdOut NewLine
;==============================================================
.section "StdOut NewLine" free
StdOut_NewLine:
	push bc
	ld bc,(stdout_col)
	ld c,STDOUT_START_COL		;reset col
	inc b						;increase row
	ld (stdout_col),bc
	pop bc
	jp StdOut_SetVRAMPointer
.ends


;==============================================================
; Sprite disabler
;==============================================================
; Sets sprite 1 to y=208
; Clobbers HL, A
;==============================================================
.section "No sprites" free
VDP_SpritesOff:
	ld hl,SpriteTableAddress | VDP_Mode_Write
	call VDP_VRAMToHL
	ld a,208
	out (VDP_Data),a
	ret
.ends

;==============================================================
; Turn on screen, wait for button, turn off screen.
;==============================================================
; Clobbers A
;==============================================================
.section "Screen Wait" free
ScreenWait:
	call VDP_ScreenOn
	call WaitForButton
	call VDP_ScreenOff
	ret
.ends

;==============================================================
; Turn on screen.
;==============================================================
; Clobbers A
;==============================================================
.section "Screen On" free
VDP_ScreenOn:
	ld a,%01100000				;$60=vblank irq
	jp VDP_SetReg1
.ends

;==============================================================
; Turn off screen.
;==============================================================
; Clobbers A
;==============================================================
.section "Screen Off" free
VDP_ScreenOff:
	ld a,$20
	jp VDP_SetReg1
.ends

;==============================================================
; Set VDP reg0.
; Clobbers A
;==============================================================
.section "Set VDP reg 0" free
VDP_SetReg0:
	out (VDP_Ctrl),a
	ld a,$80
	out (VDP_Ctrl),a
	ret
.ends

;==============================================================
; Set VDP reg1.
; Clobbers A
;==============================================================
.section "Set VDP reg 1" free
VDP_SetReg1:
	out (VDP_Ctrl),a
	ld a,$81
	out (VDP_Ctrl),a
	ret
.ends

;==============================================================
; Set VDP reg2.
; Clobbers A
;==============================================================
.section "Set VDP reg 2" free
VDP_SetReg2:
	out (VDP_Ctrl),a
	ld a,$82
	out (VDP_Ctrl),a
	ret
.ends

;==============================================================
; Set VDP reg3.
; Clobbers A
;==============================================================
.section "Set VDP reg 3" free
VDP_SetReg3:
	out (VDP_Ctrl),a
	ld a,$83
	out (VDP_Ctrl),a
	ret
.ends

;==============================================================
; Set VDP reg4.
; Clobbers A
;==============================================================
.section "Set VDP reg 4" free
VDP_SetReg4:
	out (VDP_Ctrl),a
	ld a,$84
	out (VDP_Ctrl),a
	ret
.ends

;==============================================================
; Set VDP reg5.
; Clobbers A
;==============================================================
.section "Set VDP reg 5" free
VDP_SetReg5:
	out (VDP_Ctrl),a
	ld a,$85
	out (VDP_Ctrl),a
	ret
.ends

;==============================================================
; Set VDP reg6.
; Clobbers A
;==============================================================
.section "Set VDP reg 6" free
VDP_SetReg6:
	out (VDP_Ctrl),a
	ld a,$86
	out (VDP_Ctrl),a
	ret
.ends

;==============================================================
; Set VDP reg7.
; Clobbers A
;==============================================================
.section "Set VDP reg 7" free
VDP_SetReg7:
	out (VDP_Ctrl),a
	ld a,$87
	out (VDP_Ctrl),a
	ret
.ends

;==============================================================
; Set VDP reg8.
; Clobbers A
;==============================================================
.section "Set VDP reg 8" free
VDP_SetReg8:
	out (VDP_Ctrl),a
	ld a,$88
	out (VDP_Ctrl),a
	ret
.ends

;==============================================================
; Set VDP reg9.
; Clobbers A
;==============================================================
.section "Set VDP reg 9" free
VDP_SetReg9:
	out (VDP_Ctrl),a
	ld a,$89
	out (VDP_Ctrl),a
	ret
.ends

;==============================================================
; Set VDP regA.
; Clobbers A
;==============================================================
.section "Set VDP reg A" free
VDP_SetRegA:
	out (VDP_Ctrl),a
	ld a,$8A
	out (VDP_Ctrl),a
	ret
.ends

;==============================================================
; Wait for button press
;==============================================================
; Clobbers A
; Not very efficient, I'm aiming for simplicity here
;==============================================================
.section "Wait for button press" free
WaitForButton:
-:	in a,(IO_PortA) ; get input
	cpl             ; invert bits
	or a            ; test bits
	jr nz,-         ; wait for no button press
-:	halt
	in a,(IO_PortA) ; get input
	cpl             ; invert bits
	or a            ; see if any are set
	jr z,-
	ret
WaitForNoButton:
-:	in a,(IO_PortA) ; get input
	cpl             ; invert bits
	or a            ; test bits
	jr nz,-         ; wait for no button press
	ret
.ends

;==============================================================
; Write out test results.
; hl = result text.
; a = test result (0 or FF)
;==============================================================
.section "WriteTestResult" free
WriteResult:
	push af
	call StdOut_Write
	pop af				;pop test result
	cp $00
	jr nz,ResErr
	ld hl,ok_text
	jp StdOut_Write
ResErr
	ld hl,error_text
	jp StdOut_Write

WriteResult_VCount:
	push af
	call StdOut_Write
	pop af				;pop test result
	cp $01
	jr nz,VCPALErr
	ld hl,pal_ok_text
	jp StdOut_Write
VCPALErr
	cp $02
	jr nz,VCNTSCErr
	ld hl,ntsc_ok_text
	jp StdOut_Write
VCNTSCErr
	ld hl,error_text
	jp StdOut_Write
.ends


;==============================================================
; Write out test results.
; hl = text.
; de = registervalue
;==============================================================
.section "WriteRegisterValue" free
WriteRegisterValue:
	call StdOut_Write
	ld hl,TEMP_REGISTERS
	ld b,d					;switch de around.
	ld d,e
	ld e,b
	ld (TEMP_REGISTERS),de
	ld b,2					;length chars.
	call StdOut_Write_Hex
	ld a,$0a
	jr StdOut_Char
.ends

;==============================================================
; Save all registers
;==============================================================
.section "save register" free
DumpAllRegisters:
	ld (RegisterArea+$00),sp
	ld (RegisterArea+$06),bc
	ld (RegisterArea+$08),de
	ld (RegisterArea+$0A),hl
	ld (RegisterArea+$0C),ix
	ld (RegisterArea+$0E),iy
	ld sp,$DFFE
	push af
	pop bc
	ld (RegisterArea+$04),bc	;af

	ld a,i						;affects the f-reg
	ld b,a
	ld a,r						;affects the f-reg
	ld c,a

	ld (RegisterArea+$02),bc	;ir

	ex af,af'
	push af
	pop bc
	ld (RegisterArea+$10),bc	;af'
	ex af,af'
	exx
	ld (RegisterArea+$12),bc	;bc'
	ld (RegisterArea+$14),de	;de'
	ld (RegisterArea+$16),hl	;hl'
	exx

	jp main         ; jump to main program
.ends

;==============================================================
; test1 test
;==============================================================
.section "Tests" free
RunTest1:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data1
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data1
	call CompareTestDataToVRAM

	ld (TestResult01),a
	ret

RunTest2:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data2
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	in a,(VDP_Data)
	ld hl,test_data2
	call CompareTestDataToVRAM

	ld (TestResult02),a
	ret

RunTest3:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data1
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Register
	call VDP_VRAMToHL
	in a,(VDP_Data)
	ld hl,test_data1
	call CompareTestDataToVRAM

	ld (TestResult03),a
	ret

RunTest4:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data2
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Palette
	call VDP_VRAMToHL
	in a,(VDP_Data)
	ld hl,test_data2
	call CompareTestDataToVRAM

	ld (TestResult04),a
	ret

RunTest5:
	ld hl,NameTableAddress + $5FF | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data1
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data1
	call CompareTestDataToVRAM

	ld (TestResult05),a
	ret

RunTest6:
	ld hl,NameTableAddress + $600 | VDP_Mode_Register
	call VDP_VRAMToHL
	ld hl,test_data2
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data2
	call CompareTestDataToVRAM

	ld (TestResult06),a
	ret

RunTest6_MD:
	ld hl,NameTableAddress + $610 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data3
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $610 | VDP_Mode_Register
	call VDP_VRAMToHL
	ld hl,test_data4
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $610 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data3
	call CompareTestDataToVRAM

	ld (TestResult06),a
	ret

RunTest7:
	ld hl,NameTableAddress + $610 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data3
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $610 | VDP_Mode_Palette
	call VDP_VRAMToHL
	ld hl,test_data4
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $610 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data3
	call CompareTestDataToVRAM

	ld (TestResult07),a
	ret

RunTest8:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data2
	call WriteTestDataToVRAM

	in a,(VDP_Data)
	in a,(VDP_Data)
	ld hl,test_data1
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data2
	call CompareTestDataToVRAM

	ld (TestResult08),a
	cp $00
	ret nz

	in a,(VDP_Data)
	in a,(VDP_Data)
	ld hl,test_data1
	call CompareTestDataToVRAM

	ld (TestResult08),a
	ret

RunTest9:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data1
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	in a,(VDP_Data)
	ld hl,test_data1+15
	cp (hl)
	jr nz,t9_error

	in a,(VDP_Data)
	ld hl,test_data1
	cp (hl)
	jr nz,t9_error

	ld a,$66
	out (VDP_Data),a
	nop
	nop
	ld a,$99
	nop
	nop
	in a,(VDP_Data)
	cp $66
	jr nz,t9_error

	ld a,$00
	ld (TestResult09),a
	ret

t9_error:
	ld a,$FF
	ld (TestResult09),a
	ret

RunTest9_MD:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data1
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $60F | VDP_Mode_Write
	call VDP_VRAMToHL
	in a,(VDP_Data)
	ld hl,NameTableAddress + $610 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data2
	call WriteTestDataToVRAM
	in a,(VDP_Data)
	ld hl,test_data1+15
	cp (hl)
	jr nz,t9_error_md

	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	in a,(VDP_Data)

	ld a,$66
	out (VDP_Data),a
	nop
	nop
	ld a,$99
	nop
	ld hl,test_data1
	in a,(VDP_Data)
	cp (hl)
	jr nz,t9_error_md

	ld a,$00
	ld (TestResult09),a
	ret

t9_error_md:
	ld a,$FF
	ld (TestResult09),a
	ret

RunTest10:
	ld hl,NameTableAddress + $610 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data4
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $610 | VDP_Mode_Palette
	call VDP_VRAMToHL
	in a,(VDP_Data)
	ld hl,test_data4+15
	cp (hl)
	jr nz,t10_error

	ld a,$77
	out (VDP_Data),a
	nop
	nop
	ld a,$EE
	nop
	nop
	in a,(VDP_Data)
	cp $77
	jr nz,t10_error

	ld a,$BB
	out (VDP_Data),a
	nop
	nop
	ld a,$33
	nop
	nop
	in a,(VDP_Data)
	cp $BB
	jr nz,t10_error

	ld a,$00
	ld (TestResult10),a
	ret

t10_error:
	ld a,$FF
	ld (TestResult10),a
	ret

RunTest10_MD:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data1
	call WriteTestDataToVRAM
	ld hl,test_data4
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $60F | VDP_Mode_Write
	call VDP_VRAMToHL
	in a,(VDP_Data)
	ld hl,NameTableAddress + $610 | VDP_Mode_Palette
	call VDP_VRAMToHL
	ld hl,test_data2
	call WriteTestDataToVRAM
	in a,(VDP_Data)
	ld hl,test_data1+15
	cp (hl)
	jr nz,t10_error_md

	ld hl,NameTableAddress + $610 | VDP_Mode_Palette
	call VDP_VRAMToHL
	in a,(VDP_Data)

	ld a,$66
	out (VDP_Data),a
	nop
	nop
	ld a,$99
	nop
	ld hl,test_data4
	in a,(VDP_Data)
	cp (hl)
	jr nz,t10_error_md

	ld a,$00
	ld (TestResult10),a
	ret

t10_error_md:
	ld a,$FF
	ld (TestResult10),a
	ret

RunTest11:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data4
	call WriteTestDataToVRAM
	ld hl,test_data1
	call WriteTestDataToVRAM
	ld hl,test_data3
	call WriteTestDataToVRAM
	ld hl,test_data2
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $658 | VDP_Mode_Read
	call VDP_VRAMToHL

	ld a,$10
	out (VDP_Ctrl),a		;Set low part of address
	in a,(VDP_Data)
	ld hl,test_data1
	call CompareTestDataToVRAM
	ld (TestResult11),a
	cp $00
	ret nz

	ld a,$30
	out (VDP_Ctrl),a		;Set low part of address
	in a,(VDP_Data)
	ld hl,test_data2
	call CompareTestDataToVRAM
	ld (TestResult11),a
	cp $00
	ret nz

	ld a,$20
	out (VDP_Ctrl),a		;Set low part of address
	in a,(VDP_Data)
	ld hl,test_data3
	call CompareTestDataToVRAM
	ld (TestResult11),a
	cp $00
	ret nz

	ld a,$00
	out (VDP_Ctrl),a		;Set low part of address
	in a,(VDP_Data)
	ld hl,test_data4
	call CompareTestDataToVRAM
	ld (TestResult11),a
	ret

RunTest11_MD:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data4
	call WriteTestDataToVRAM
	ld hl,test_data1
	call WriteTestDataToVRAM
	ld hl,test_data3
	call WriteTestDataToVRAM
	ld hl,test_data2
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL

	ld a,$10
	out (VDP_Ctrl),a		;Set low part of address
	ld hl,test_data4
	call CompareTestDataToVRAM
	ld (TestResult11),a
	cp $00
	ret nz

	ld a,$30
	out (VDP_Ctrl),a		;Set low part of address
	ld hl,test_data1
	call CompareTestDataToVRAM
	ld (TestResult11),a
	cp $00
	ret nz

	ld a,$20
	out (VDP_Ctrl),a		;Set low part of address
	ld hl,test_data3
	call CompareTestDataToVRAM
	ld (TestResult11),a
	cp $00
	ret nz

	ld a,$00
	out (VDP_Ctrl),a		;Set low part of address
	ld hl,test_data2
	call CompareTestDataToVRAM
	ld (TestResult11),a
	ret

RunTest12:
	ld hl,NameTableAddress + $672 | VDP_Mode_Write
	call VDP_VRAMToHL

	ld a,$30
	out (VDP_Ctrl),a		;Set low part of address
	ld hl,test_data4
	call WriteTestDataToVRAM
	ld a,$00
	out (VDP_Ctrl),a		;Set low part of address
	ld hl,test_data1
	call WriteTestDataToVRAM
	ld a,$20
	out (VDP_Ctrl),a		;Set low part of address
	ld hl,test_data3
	call WriteTestDataToVRAM
	ld a,$10
	out (VDP_Ctrl),a		;Set low part of address
	ld hl,test_data2
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL

	ld hl,test_data1
	call CompareTestDataToVRAM
	ld (TestResult12),a
	cp $00
	ret nz

	ld hl,test_data2
	call CompareTestDataToVRAM
	ld (TestResult12),a
	cp $00
	ret nz

	ld hl,test_data3
	call CompareTestDataToVRAM
	ld (TestResult12),a
	cp $00
	ret nz

	ld hl,test_data4
	call CompareTestDataToVRAM
	ld (TestResult12),a
	ret

RunTest12_MD:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL

	ld a,$30
	out (VDP_Ctrl),a		;Set low part of address
	ld hl,test_data1
	call WriteTestDataToVRAM
	ld a,$00
	out (VDP_Ctrl),a		;Set low part of address
	ld hl,test_data2
	call WriteTestDataToVRAM
	ld a,$20
	out (VDP_Ctrl),a		;Set low part of address
	ld hl,test_data3
	call WriteTestDataToVRAM
	ld a,$10
	out (VDP_Ctrl),a		;Set low part of address
	ld hl,test_data4
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL

	ld hl,test_data1
	call CompareTestDataToVRAM
	ld (TestResult12),a
	cp $00
	ret nz

	ld hl,test_data2
	call CompareTestDataToVRAM
	ld (TestResult12),a
	cp $00
	ret nz

	ld hl,test_data3
	call CompareTestDataToVRAM
	ld (TestResult12),a
	cp $00
	ret nz

	ld hl,test_data4
	call CompareTestDataToVRAM
	ld (TestResult12),a
	ret

RunTest13:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data3
	call WriteTestDataToVRAM
	ld hl,test_data2
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $610 | VDP_Mode_Palette
	call VDP_VRAMToHL
	ld a,$10
	out (VDP_Ctrl),a
	ld hl,test_data1
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data3
	call CompareTestDataToVRAM
	ld (TestResult13),a
	cp $00
	ret nz

	ld hl,test_data2
	call CompareTestDataToVRAM
	ld (TestResult13),a
	ret

RunTest14:
	ld hl,$3FF0 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data1
	call WriteTestDataToVRAM
	ld hl,test_data4
	call WriteTestDataToVRAM

	ld hl,$3FF0 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data1
	call CompareTestDataToVRAM
	ld (TestResult14),a
	cp $00
	ret nz

	ld hl,test_data4
	call CompareTestDataToVRAM
	ld (TestResult14),a
	cp $00
	ret nz

	ld hl,$0000 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data4
	call CompareTestDataToVRAM
	ld (TestResult14),a

	ld hl,$3FF0 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld a,$00
	ld hl,$0020
	call VDP_FillVRAM
	ret

RunTest15:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data3
	call WriteTestDataToVRAM
	ld hl,test_data2
	call WriteTestDataToVRAM
	ld hl,test_data1
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data3
	call CompareTestDataToVRAM
	ld (TestResult15),a
	cp $00
	ret nz

	ld hl,NameTableAddress + $610 | VDP_Mode_Palette
	call VDP_VRAMToHL
	ld hl,test_data4
	call WriteTestDataToVRAM

	in a,(VDP_Data)
	ld hl,test_data1
	call CompareTestDataToVRAM
	ld (TestResult15),a
	ret

RunTest16:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data2
	call WriteTestDataToVRAM
	ld hl,test_data3
	call WriteTestDataToVRAM

	ld a,$11
	out (VDP_Ctrl),a		;Set low part of address
	nop
	in a,(VDP_Ctrl)			;Reset latch
	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data2
	call CompareTestDataToVRAM
	ld (TestResult16),a
	ret

RunTest17:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data4
	call WriteTestDataToVRAM
	ld hl,test_data1
	call WriteTestDataToVRAM

	ld a,$1F
	out (VDP_Ctrl),a		;Set low part of address
	nop
	in a,(VDP_Data)			;Reset latch
	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data4
	call CompareTestDataToVRAM
	ld (TestResult17),a
	ret

RunTest18:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data1
	call WriteTestDataToVRAM
	ld hl,test_data3
	call WriteTestDataToVRAM

	ld a,$04
	out (VDP_Ctrl),a		;Set low part of address
	nop
	out (VDP_Data),a		;Reset latch
	ld hl,NameTableAddress + $610 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data3
	call CompareTestDataToVRAM
	ld (TestResult18),a
	ret

;==============================================================
RunTest19:
	call SetUnknownRegsToCrap
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data1
	call WriteTestDataToVRAM

	call SetUnknownRegsToNormal
	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data1
	call CompareTestDataToVRAM
	ld (TestResult19),a
	cp $00
	ret nz

	call SetUnknownRegsToNormal
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data2
	call WriteTestDataToVRAM

	call SetUnknownRegsToCrap
	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data2
	call CompareTestDataToVRAM
	ld (TestResult19),a
	ret

SetUnknownRegsToCrap:
	ld hl,$8B04				; mode reg3 = 4
	call VDP_VRAMToHL
	ld hl,$8C04				; mode reg4 = 4
	call VDP_VRAMToHL
	ld hl,$8D04				; hscroll tbl adr = 4
	call VDP_VRAMToHL
	ld hl,$8E04				; unknown = 4
	call VDP_VRAMToHL
	ld hl,$8F04				; autoincrement = 4
	call VDP_VRAMToHL
	ret
SetUnknownRegsToNormal:
	ld hl,$8B00				; mode reg3 = 0
	call VDP_VRAMToHL
	ld hl,$8C00				; mode reg4 = 0
	call VDP_VRAMToHL
	ld hl,$8D00				; hscroll tbl adr = 0
	call VDP_VRAMToHL
	ld hl,$8E00				; unknown = 0
	call VDP_VRAMToHL
	ld hl,$8F01				; autoincrement = 1
	call VDP_VRAMToHL
	ret

;==============================================================
RunTest21:
	ld c,VCounter
	call VCount_Test
	ld (TestResult21),a
	ret
	
VCount_Test
	ld de,320
-:	in a,(c)
	cp $ff
	jr z,saveVertical
	cp b
	jr z,-
	ld b,a
	dec de
	ld a,d
	or e
	jr nz,-

saveVertical:
	ld hl,TestAreaVC
	ld de,313
	ld b,a
-:	in a,(VCounter)			;Read VCounter values
	cp b
	jr z,-
	ld b,a
	ld (hl),a
	inc hl
	dec de
	ld a,d
	or e
	jr nz,-

checkVertical:
	ld de,313
	ld hl,TestAreaVC
	exx
	ld hl,VCounterValues
	exx
-:	exx
	ld a,(hl)
	inc hl
	exx
	cp (hl)
	jr nz,checkVerticalNTSC
	inc hl
	dec de
	ld a,d
	or e
	jr nz,-

	ld a,$01				; PAL OK
	ret

checkVerticalNTSC:
	exx
	ld hl,VCounterNTSC
	exx
	ld a,e
	sub 51
	ld e,a
-:	exx
	ld a,(hl)
	inc hl
	exx
	cp (hl)
	jr nz,t21_error
	inc hl
	dec de
	ld a,d
	or e
	jr nz,-

	ld a,$02				; NTSC OK
	ret

t21_error:
	ld a,$FF				;VCount Error
	ret


;==============================================================
RunTest22:
	ld b,254
	in a,(HCounter)
	ld d,a
-:	in a,(HCounter)
	cp d
	jr nz,t22_error
	djnz -

	ld a,$00				; OK
	ld (TestResult22),a
	ret

t22_error:
	ld a,$FF				;HCount Error
	ld (TestResult22),a
	ret


;==============================================================
RunTest23:
	ld c,HCounter
	ld b,254
	call Do_hc_test_2
	ld (TestResult23),a
	ret


Do_hc_test_2
	ld a,$05				;IO ports set to output low
	out (IO_Ctrl),a
	in a,(c)
	ld d,a
-:	in a,(c)
	cp d
	jr nz,t23_error
	ld a,$FF				;flip HL in/out so HCounter gets latched.
	out (IO_Ctrl),a
	in a,(c)
	cp d
	jr z,t23_error
	ld d,a
	ld a,$05				;IO ports set to output low
	out (IO_Ctrl),a
	djnz -

	ld a,$00				; OK
	ret

t23_error:
	ld a,$FF				;HCount Error
	ret


;==============================================================
RunTest24:
	ld hl,TestAreaHC
	ld b,232
-:	ld a,$55				;7 cycles
	out (IO_Ctrl),a			;11 cycles
	ld a,$FF				;7 cycles, flip HL in/out so HCounter gets latched.
	out (IO_Ctrl),a			;11 cycles
	in a,(HCounter)			;Read HCounter value, loop 229 cycles
	ld (hl),a
	inc hl

	ld de,($C000)			;20 cycles each
	ld de,($C000)
	ld de,($C000)
	ld de,($C000)
	ld de,($C000)
	ld de,($C000)
	ld de,($C000)
	ld de,$C000				;10 cycles
	dec de					;6 cycles

	djnz -

	ld b,228
	exx
	ld hl,TestAreaHC
	ld a,(hl)
	inc hl
	cp (hl)
	jr z,skip_repeated_value
	dec hl
skip_repeated_value
	exx
	ld hl,HCounterValues-1
-:	inc hl
	cp (hl)
	jr z,+
	djnz -
+:
	inc hl
	cp (hl)
	jr z,skip_repeated_value_2
	dec hl
skip_repeated_value_2

	ld b,229
-:	exx
	ld a,(hl)
	inc hl
	exx
	cp (hl)
	jr nz,t24_error
	inc hl
	djnz -

	ld a,$00
	ld (TestResult24),a
	ret

t24_error:
	ld a,$FF
	ld (TestResult24),a
	ret

	
;==============================================================
RunTest25:
	in a,(VDP_Ctrl)			;Clear any pending irqs.
	ei
	halt					;wait for vblank
	ld hl,$8014				; hblank on
	call VDP_VRAMToHL
	ld hl,$8A80				; hblank row $80
	call VDP_VRAMToHL
	ld a,$00
	ld (Timer2),a

	halt					;wait for hblank
	ld hl,$8004				; hblank off
	call VDP_VRAMToHL
	ld a,(Timer2)
	cp $00
	jr z,t25_error
;---------------	
	halt					;wait for vblank
	ld hl,$9014				; hblank on (mirror 1 of reg0)
	call VDP_VRAMToHL
	ld a,$00
	ld (Timer2),a

	halt					;wait for hblank
	ld hl,$8004				; hblank off
	call VDP_VRAMToHL
	ld a,(Timer2)
	cp $00
	jr z,t25_error
;---------------	
	halt					;wait for vblank
	ld hl,$A014				; hblank on (mirror 2 of reg0)
	call VDP_VRAMToHL
	ld a,$00
	ld (Timer2),a

	halt					;wait for hblank
	ld hl,$8004				; hblank off
	call VDP_VRAMToHL
	ld a,(Timer2)
	cp $00
	jr z,t25_error
;---------------	
	halt					;wait for vblank
	ld hl,$B014				; hblank on (mirror 3 of reg0)
	call VDP_VRAMToHL
	ld a,$00
	ld (Timer2),a

	halt					;wait for hblank
	ld hl,$8004				; hblank off
	call VDP_VRAMToHL
	ld a,(Timer2)
	cp $00
	jr z,t25_error
;---------------	

	ld hl,$8AFF				; hblank row $FF
	call VDP_VRAMToHL
	ld a,$00
	ld (TestResult25),a
	di
	ret

t25_error:
	ld hl,$8AFF				; hblank row $FF
	call VDP_VRAMToHL
	ld a,$FF
	ld (TestResult25),a
	di
	ret


RunTest25_MD:
	in a,(VDP_Ctrl)			;Clear any pending irqs.
	ei
	halt					;wait for vblank
	ld hl,$8014				; hblank on
	call VDP_VRAMToHL
	ld hl,$8A80				; hblank row $80
	call VDP_VRAMToHL
	ld a,$00
	ld (Timer2),a

	halt					;wait for hblank
	ld hl,$8004				; hblank off
	call VDP_VRAMToHL
	ld a,(Timer2)
	cp $00
	jr z,t25_error_md
;---------------	
	halt					;wait for vblank
	ld hl,$A014				; hblank on (mirror of reg0)
	call VDP_VRAMToHL
	ld a,$00
	ld (Timer2),a

	halt					;wait for hblank
	ld hl,$8004				; hblank off
	call VDP_VRAMToHL
	ld a,(Timer2)
	cp $00
	jr z,t25_error_md
;---------------	
	halt					;wait for vblank
	ld hl,$9014				; hblank on (not mirror of reg0 on MD)
	call VDP_VRAMToHL
	ld a,$00
	ld (Timer2),a

	halt					;wait for hblank
	ld hl,$8004				; hblank off
	call VDP_VRAMToHL
	ld a,(Timer2)
	cp $00
	jr nz,t25_error_md
;---------------	
	halt					;wait for vblank
	ld hl,$B014				; hblank on (not mirror of reg0 on MD)
	call VDP_VRAMToHL
	ld a,$00
	ld (Timer2),a

	halt					;wait for hblank
	ld hl,$8004				; hblank off
	call VDP_VRAMToHL
	ld a,(Timer2)
	cp $00
	jr nz,t25_error_md
;---------------	


	ld hl,$8AFF				; hblank row $FF
	call VDP_VRAMToHL
	ld a,$00
	ld (TestResult25),a
	di
	ret


t25_error_md:
	ld hl,$8AFF				; hblank row $FF
	call VDP_VRAMToHL
	ld a,$FF
	ld (TestResult25),a
	di
	ret

;==============================================================
RunTest26:
	ld c,$80
-:	call t26_test_vdp_ports
	cp $00
	jr nz,t26_error
	inc c
	inc c
	ld a,c
	cp $BE
	jr nz,-
	ld a,$00
	ld (TestResult26),a
	ret

t26_error:
	ld (TestResult26),a
	ret

t26_test_vdp_ports:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data1
	call T26_WriteToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data1
	call T26_CompareToVRAM
	cp $00
	jr nz,t26_error

	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,test_data2
	call T26_WriteToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call VDP_VRAMToHL
	ld hl,test_data2
	call T26_CompareToVRAM
	ret

T26_WriteToVRAM:
	ld b,$10
-:	ld a,(hl)
	out (c),a
	inc hl
	djnz -
	ret

T26_CompareToVRAM:
	ld b,$10
-:	in a,(c)
	cp (hl)
	jr nz,t26_cp_error
	inc hl
	djnz -
	ld a,$00
	ret
t26_cp_error:
	ld a,$FF
	ret

;==============================================================
RunTest27:
	ld c,$81
-:	call t27_test_vdp_ports
	cp $00
	jr nz,t27_error
	inc c
	inc c
	ld a,c
	cp $BF
	jr nz,-
	ld a,$00
	ld (TestResult27),a
	ret

t27_error:
	ld (TestResult27),a
	ret

t27_test_vdp_ports:
	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call T27_VDP_VRAMToHL
	ld hl,test_data1
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call T27_VDP_VRAMToHL
	ld hl,test_data1
	call CompareTestDataToVRAM
	cp $00
	jr nz,t27_error

	ld hl,NameTableAddress + $600 | VDP_Mode_Write
	call T27_VDP_VRAMToHL
	ld hl,test_data2
	call WriteTestDataToVRAM

	ld hl,NameTableAddress + $600 | VDP_Mode_Read
	call T27_VDP_VRAMToHL
	ld hl,test_data2
	call CompareTestDataToVRAM
	ret

T27_VDP_VRAMToHL:
	push af
		ld a,l
		out (c),a
		ld a,h
		out (c),a
	pop af
	ret


;==============================================================
RunTest28:
	ld c,$40
-:	call VCount_Test
	cp $FF
	jr z,t28_error
	inc c
	inc c
	ld a,c
	cp $7E
	jr nz,-
	ld a,$00
	ld (TestResult28),a
	ret
t28_error:
	ld (TestResult28),a
	ret

;==============================================================
RunTest29:
	ld c,$41
-:	ld b,127
	call Do_hc_test_2
	cp $00
	jr nz,t29_error
	inc c
	inc c
	ld a,c
	cp $7F
	jr nz,-
	ld a,$00
	ld (TestResult29),a
	ret
t29_error:
	ld (TestResult29),a
	ret

	
;==============================================================
WriteTestDataToVRAM:
	ld b,$10
	jp VDP_WriteToVRAM_Short
	
CompareTestDataToVRAM:
	ld b,$10
-:	in a,(VDP_Data)
	cp (hl)
	jr nz,cp_error_1
	inc hl
	djnz -
	ld a,$00
	ret
cp_error_1:
	ld a,$FF
	ret

.ends

;==============================================================
; Switch to mode 5, set palette, switch to mode 4 to se palette
;==============================================================
.section "MD Palette test" FREE
StartMDPaletteTest:
	ld hl,(NameTableAddress + 64*2) | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,cubes_data				; data
	ld bc,$40						; size
	call VDP_WriteToVRAM

	ld hl,(NameTableAddress + 64*3) | VDP_Mode_Write
	call VDP_VRAMToHL
	ld hl,cubes_data				; data
	ld bc,$40						; size
	call VDP_WriteToVRAM

	ld a,$24
	call VDP_SetReg1				;turn on mode5

md_palette_setup:
	ld hl,$C000
	call VDP_VRAMToHL
	ld hl,$0000
	call VDP_VRAMToHL
	ld hl,cubes_md_palette
	ld bc,$20BE
md_pal_loop:
	ld a,(hl)
	inc hl
	out (c),a
	djnz md_pal_loop

	ld a,$64
	call VDP_SetReg1				;screen on, mode5.
	ei
	call WaitForButton

	ld a,$60
	call VDP_SetReg1				;turn off mode5

;	call VDP_ScreenOn
;	ei
	call WaitForButton
	jp WarmReset

.ends

;==============================================================
; Data
;==============================================================
;.BANK 4 SLOT 2
.section "Data1" FREE
	menu_text:
	.db "  Please select a test!",$0a,$0a,$0a
	.db "  SMS VDP Test.",$0a
	.db "  MegaDrive VDP Test.",$0a
	.db "  MegaDrive Palette Test.",$0a
	.db "  Register Startup Values.",$0a
	.db "  CPU Test.",$0a
	.db 0

	space_text:
	.db " ",$00

	marker_text:
	.db "*",$00

	continue_text:
	.db $0a,$0a,"Push any button for next tests",0

sms_test1_text:
	.db "SMS VDP data test",$0a,$0a,0
sms_test2_text:
	.db "SMS VDP misc test",$0a,$0a,0
md_test1_text:
	.db "MegaDrive VDP data test",$0a,$0a,0
md_test2_text:
	.db "MegaDrive VDP misc test",$0a,$0a,0
md_page3_text
	.db "MegaDrive HCounter Values",$0a,0
cpu_regs_text:
	.db "CPU Registers Startup Values",$0a,$0a,0

	res_text_1:
	.db "Normal read/write:     ",0
	res_text_2:
	.db "Read after code1 wr:   ",0
	res_text_3:
	.db "Read after code2 wr:   ",0
	res_text_4:
	.db "Read after code3 wr:   ",0
	res_text_5:
	.db "Write after code0 wr:  ",0
	res_text_6:
	.db "Write after code2 wr:  ",0
	res_text_7:
	.db "Write after code3 wr:  ",0
	res_text_8:
	.db "Mixed read/write:      ",0
	res_text_9:
	.db "VRAM wr set VDPbuffer: ",0
	res_text_9_md:
	.db "VRAM wr keep VDPbuffer:",0
	res_text_10:
	.db "CRAM wr set VDPbuffer: ",0
	res_text_10_md:
	.db "CRAM wr keep VDPbuffer:",0
	res_text_11:
	.db "1byte wr sets rd adr:  ",0
	res_text_11_md:
	.db "1byte wr keeps rd adr: ",0
	res_text_12:
	.db "1byte wr sets wr adr:  ",0
	res_text_12_md:
	.db "1byte wr keeps wr adr: ",0
	res_text_13:
	.db "1byte wr keeps wr mode:",0
	res_text_14:
	.db "Address wraps at $3FFF:",0
	res_text_15:
	.db "VRAM & CRAM share adr: ",0
	res_text_16:
	.db "Rd VDPCtrl reset latch:",0
	res_text_17:
	.db "Rd VDPData reset latch:",0
	res_text_18:
	.db "Wr VDPData reset latch:",0
	res_text_19:
	.db "Unused regs, no effect:",0

	res_text_21:
	.db "VCounter values:",0
	res_text_22:
	.db "HCounter keeps value:",0
	res_text_23:
	.db "HC change on TH 0->1:",0
	res_text_24:
	.db "HCounter correct:    ",0
	res_text_25:
	.db "VDP Register mirrors:",0
	res_text_26:
	.db "VDP data mirrors:    ",0
	res_text_27:
	.db "VDP ctrl mirrors:    ",0
	res_text_28:
	.db "VCounter mirrors:    ",0
	res_text_29:
	.db "HCounter mirrors:    ",0


reg_text_sp:
	.db "Register SP:  $",0
reg_text_ir:
	.db "Register IR:  $",0
reg_text_af:
	.db "Register AF:  $",0
reg_text_bc:
	.db "Register BC:  $",0
reg_text_de:
	.db "Register DE:  $",0
reg_text_hl:
	.db "Register HL:  $",0
reg_text_ix:
	.db "Register IX:  $",0
reg_text_iy:
	.db "Register IY:  $",0
reg_text_af2:
	.db "Register AF': $",0
reg_text_bc2:
	.db "Register BC': $",0
reg_text_de2:
	.db "Register DE': $",0
reg_text_hl2:
	.db "Register HL': $",0



ok_text:
	.db " Ok.",$0a,0
error_text:
	.db " Error.",$0a,0
pal_ok_text:
	.db "  PAL Ok.",$0a,0
ntsc_ok_text:
	.db " NTSC Ok.",$0a,0

cubes_data:
	.dw $00,$00,$60,$60,$61,$61,$62,$62,$63,$63,$64,$64,$65,$65,$66,$66
	.dw $67,$67,$68,$68,$69,$69,$6a,$6a,$6b,$6b,$6c,$6c,$6d,$6d,$6e,$6e
cubes_md_palette:
	.dw $000D,$0EEE,$0CCC,$0AAA,$0888,$0666,$0444,$0222,$0000,$00E0,$00C0,$00A0,$0080,$0060,$0040,$0020

test_data1:
	.db $00,$FF,$55,$AA,$01,$02,$04,$08,$10,$20,$40,$80,"Test"
test_data2:
	.db "TESt",$AA,$55,$FF,$00,$80,$40,$20,$10,$08,$04,$02,$01
test_data3:
	.db "GuefsPoaLeYit",$AA,$55,$00
test_data4:
	.db $AA,$00,$55,"TeHuGuefsPoaL"


HCounterValues:				;MegaDrive can have double values in all places (eg FF or 00 or 01).
	.db $00,$01,$02,$02,$03,$04,$05,$05,$06,$07,$08,$08,$09,$0A,$0B,$0B
	.db $0C,$0D,$0E,$0E,$0F,$10,$11,$11,$12,$13,$14,$14,$15,$16,$17,$17
	.db $18,$19,$1A,$1A,$1B,$1C,$1D,$1D,$1E,$1F,$20,$20,$21,$22,$23,$23
	.db $24,$25,$26,$26,$27,$28,$29,$29,$2A,$2B,$2C,$2C,$2D,$2E,$2F,$2F
	.db $30,$31,$32,$32,$33,$34,$35,$35,$36,$37,$38,$38,$39,$3A,$3B,$3B
	.db $3C,$3D,$3E,$3E,$3F,$40,$41,$41,$42,$43,$44,$44,$45,$46,$47,$47
	.db $48,$49,$4A,$4A,$4B,$4C,$4D,$4D,$4E,$4F,$50,$50,$51,$52,$53,$53
	.db $54,$55,$56,$56,$57,$58,$59,$59,$5A,$5B,$5C,$5C,$5D,$5E,$5F,$5F
	.db $60,$61,$62,$62,$63,$64,$65,$65,$66,$67,$68,$68,$69,$6A,$6B,$6B
	.db $6C,$6D,$6E,$6E,$6F,$70,$71,$71,$72,$73,$74,$74,$75,$76,$77,$77
	.db $78,$79,$7A,$7A,$7B,$7C,$7D,$7D,$7E,$7F,$80,$80,$81,$82,$83,$83
	.db $84,$85,$86,$86,$87,$88,$89,$89,$8A,$8B,$8C,$8C,$8D,$8E,$8F,$8F
	.db $90,$91,$92,$92,$93,$E9,$EA,$EA,$EB,$EC,$ED,$ED,$EE,$EF,$F0,$F0
	.db $F1,$F2,$F3,$F3,$F4,$F5,$F6,$F6,$F7,$F8,$F9,$F9,$FA,$FB,$FC,$FC
	.db $FD,$FE,$FF,$FF

	.db $00,$01,$02,$02,$03,$04,$05,$05,$06,$07,$08,$08,$09,$0A,$0B,$0B
	.db $0C,$0D,$0E,$0E,$0F,$10,$11,$11,$12,$13,$14,$14,$15,$16,$17,$17
	.db $18,$19,$1A,$1A,$1B,$1C,$1D,$1D,$1E,$1F,$20,$20,$21,$22,$23,$23
	.db $24,$25,$26,$26,$27,$28,$29,$29,$2A,$2B,$2C,$2C,$2D,$2E,$2F,$2F
	.db $30,$31,$32,$32,$33,$34,$35,$35,$36,$37,$38,$38,$39,$3A,$3B,$3B
	.db $3C,$3D,$3E,$3E,$3F,$40,$41,$41,$42,$43,$44,$44,$45,$46,$47,$47
	.db $48,$49,$4A,$4A,$4B,$4C,$4D,$4D,$4E,$4F,$50,$50,$51,$52,$53,$53
	.db $54,$55,$56,$56,$57,$58,$59,$59,$5A,$5B,$5C,$5C,$5D,$5E,$5F,$5F
	.db $60,$61,$62,$62,$63,$64,$65,$65,$66,$67,$68,$68,$69,$6A,$6B,$6B
	.db $6C,$6D,$6E,$6E,$6F,$70,$71,$71,$72,$73,$74,$74,$75,$76,$77,$77
	.db $78,$79,$7A,$7A,$7B,$7C,$7D,$7D,$7E,$7F,$80,$80,$81,$82,$83,$83
	.db $84,$85,$86,$86,$87,$88,$89,$89,$8A,$8B,$8C,$8C,$8D,$8E,$8F,$8F
	.db $90,$91,$92,$92,$93,$E9,$EA,$EA,$EB,$EC,$ED,$ED,$EE,$EF,$F0,$F0
	.db $F1,$F2,$F3,$F3,$F4,$F5,$F6,$F6,$F7,$F8,$F9,$F9,$FA,$FB,$FC,$FC
	.db $FD,$FE,$FF,$FF

VCounterValues:
	.db $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F
	.db $10,$11,$12,$13,$14,$15,$16,$17,$18,$19,$1A,$1B,$1C,$1D,$1E,$1F
	.db $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F
	.db $30,$31,$32,$33,$34,$35,$36,$37,$38,$39,$3A,$3B,$3C,$3D,$3E,$3F
	.db $40,$41,$42,$43,$44,$45,$46,$47,$48,$49,$4A,$4B,$4C,$4D,$4E,$4F
	.db $50,$51,$52,$53,$54,$55,$56,$57,$58,$59,$5A,$5B,$5C,$5D,$5E,$5F
	.db $60,$61,$62,$63,$64,$65,$66,$67,$68,$69,$6A,$6B,$6C,$6D,$6E,$6F
	.db $70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$7A,$7B,$7C,$7D,$7E,$7F
	.db $80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F
	.db $90,$91,$92,$93,$94,$95,$96,$97,$98,$99,$9A,$9B,$9C,$9D,$9E,$9F
	.db $A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7,$A8,$A9,$AA,$AB,$AC,$AD,$AE,$AF
	.db $B0,$B1,$B2,$B3,$B4,$B5,$B6,$B7,$B8,$B9,$BA,$BB,$BC,$BD,$BE,$BF
	.db $C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF
	.db $D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
	.db $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
	.db $F0,$F1,$F2
VCounterPAL:
	.db $BA,$BB,$BC,$BD,$BE,$BF
	.db $C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF
	.db $D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
	.db $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
	.db $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF
VCounterNTSC:
	.db $D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
	.db $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
	.db $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF


	chartiles:
	.incbin "smscharset (tiles).pscompr"
	charpalette:
	.incbin "smscharset (palette).bin" fsize charpalettesize
.ends



